@@ -7,8 +7,6 @@
 # [a-z]+
 
 # TODO 处理好和纵向滚动条的相对位置问题
-# TODO shell中的stdout无法匹配
-# TODO pat == '' 时报错提示
 
 
 if __name__ == '__main__':
@@ -115,12 +113,17 @@ class ReplaceBar(tk.Frame):
         self.text.tag_remove('hit', '1.0', 'end')
         self.tip.config(text=' Match: 0')
 
-        if not self.engine.getpat():
+        # banning show error report temporarily
+        _report_error = self.engine.report_error
+        self.engine.report_error = lambda *v: None
+        prog = self.engine.getprog()
+        self.engine.report_error = _report_error
+
+        if not prog:
             return
 
         s = self.text.get('1.0', 'end-1c')
         insert = len(self.text.get('1.0', 'insert'))
-        prog = self.engine.getprog()
         matchs = [m.span() for m in prog.finditer(s)]
         self.tip.config(text=' Match: %d' % len(matchs))
         for n, (p1, p2) in enumerate(matchs):
@@ -139,6 +142,20 @@ class ReplaceBar(tk.Frame):
         self.text.after(5, self.Update) # TODO unknown reason
 
     def ReplaceAll(self):
-        # TODO 替换选区部分内容
-        self.replace.replace_all()
+        text = self.text
+        if not text.tag_ranges('sel'):
+            self.replace.replace_all()
+        else:
+            self.engine.wrapvar.set(False)
+            text.undo_block_start()
+
+            tail = text.get('sel.last', 'end-1c')
+            text.delete('sel.last', 'end-1c')
+            text.tag_remove('sel', '1.0', 'end')
+            self.replace.replace_all()
+            text.insert('end-1c', tail) # TODO 撤销/重做时光标会移动到最后
+
+            text.undo_block_stop()
+            self.engine.wrapvar.set(True)
+
         self.text.after(5, self.Update) # TODO unknown reason
